#!/usr/bin/env python
# -*- coding:utf-8 -*-

from __future__ import print_function, unicode_literals
import unittest

from zenfeed.fetcher import (
    sanitize_url, parse_url, concat_urls, default_favicon_url
)

class TestParseUrl(unittest.TestCase):

    def test_parse_url_simple(self):
        # Arrange
        url = 'http://hveem.no'
        expected_url_begin = 'http://hveem.no'
        expected_url_host = 'http://hveem.no'
        # Act
        url_begin, url_host = parse_url(url)
        # Assert
        self.assertEqual(url_begin, expected_url_begin)
        self.assertEqual(url_host, expected_url_host)

    def test_parse_url_simple_with_trailing_slash(self):
        # Arrange
        url = 'http://hveem.no/'
        expected_url_begin = 'http://hveem.no/'
        expected_url_host = 'http://hveem.no'
        # Act
        url_begin, url_host = parse_url(url)
        # Assert
        self.assertEqual(url_begin, expected_url_begin)
        self.assertEqual(url_host, expected_url_host)

    def test_parse_url(self):
        # Arrange
        url = 'http://hveem.no/feed/'
        expected_url_begin = 'http://hveem.no/feed/'
        expected_url_host = 'http://hveem.no'
        # Act
        url_begin, url_host = parse_url(url)
        # Assert
        self.assertEqual(url_begin, expected_url_begin)
        self.assertEqual(url_host, expected_url_host)

    def test_parse_url_complicated(self):
        # Arrange
        url = 'http://hveem.no/feed/foo?baz=42&sort#hash'
        expected_url_begin = 'http://hveem.no/feed/foo'
        expected_url_host = 'http://hveem.no'
        # Act
        url_begin, url_host = parse_url(url)
        # Assert
        self.assertEqual(url_begin, expected_url_begin)
        self.assertEqual(url_host, expected_url_host)


class TestSanitizeUrl(unittest.TestCase):

    def test_sanitize_url_https(self):
        # Arrange
        url = 'https://hveem.no/foo?q=bar#hash'
        expected_sanitized_url = 'https://hveem.no/foo?q=bar#hash'
        # Act
        sanitized_url = sanitize_url(url)
        # Assert
        self.assertEqual(sanitized_url, expected_sanitized_url)

    def test_sanitize_url_missing_http(self):
        # Arrange
        url = 'hveem.no/foo?q=bar#hash'
        expected_sanitized_url = 'http://hveem.no/foo?q=bar#hash'
        # Act
        sanitized_url = sanitize_url(url)
        # Assert
        self.assertEqual(sanitized_url, expected_sanitized_url)


class TestConcatUrl(unittest.TestCase):

    def test_concat_url_simple_absolute(self):
        # Arrange
        url = 'http://hveem.no/foo/'
        path = '/index.html'
        expected_sum_url = 'http://hveem.no/index.html'
        # Act
        sum_url = concat_urls(url, path)
        # Assert
        self.assertEqual(sum_url, expected_sum_url)

    def test_concat_url_simple_relative(self):
        # Arrange
        url = 'http://hveem.no/foo/bar'
        path = 'baz/index.html'
        expected_sum_url = 'http://hveem.no/foo/bar/baz/index.html'
        # Act
        sum_url = concat_urls(url, path)
        # Assert
        self.assertEqual(sum_url, expected_sum_url)

    def test_concat_url_simple_relative_with_trailing_slash(self):
        # Arrange
        url = 'http://hveem.no/foo/'
        path = 'index.html'
        expected_sum_url = 'http://hveem.no/foo/index.html'
        # Act
        sum_url = concat_urls(url, path)
        # Assert
        self.assertEqual(sum_url, expected_sum_url)

    def test_concat_url_complicated(self):
        # Arrange
        url = 'http://hveem.no/foo/bar?q=baz'
        path = '/fizz/logo.png'
        expected_sum_url = 'http://hveem.no/fizz/logo.png'
        # Act
        sum_url = concat_urls(url, path)
        # Assert
        self.assertEqual(sum_url, expected_sum_url)


class TestDefaultFaviconUrl(unittest.TestCase):

    def setUp(self):
        self.favicon_url_expected = 'http://hveem.no/favicon.ico'

    def test_default_favicon_url_simple(self):
        # Arrange
        url = 'http://hveem.no'
        # Act
        favicon_url = default_favicon_url(url)
        # Assert
        self.assertEqual(favicon_url, self.favicon_url_expected)

    def test_default_favicon_url_trailing_slash(self):
        # Arrange
        url = 'http://hveem.no/'
        # Act
        favicon_url = default_favicon_url(url)
        # Assert
        self.assertEqual(favicon_url, self.favicon_url_expected)

    def test_default_favicon_url_complex(self):
        # Arrange
        url = 'http://hveem.no/feed'
        # Act
        favicon_url = default_favicon_url(url)
        # Assert
        self.assertEqual(favicon_url, self.favicon_url_expected)

    def test_default_favicon_url_complicated(self):
        # Arrange
        url = 'http://hveem.no/feed/foo/bar?baz=42&sort#hash'
        # Act
        favicon_url = default_favicon_url(url)
        # Assert
        self.assertEqual(favicon_url, self.favicon_url_expected)


if __name__ == '__main__':
    unittest.main()




#!/usr/bin/env python
# -*- coding:utf-8 -*-

from __future__ import unicode_literals, print_function

from log import logger
import models


def get_step_list(from_version):
    # Zenfeed versions introducing migrations
    versions = [
        '0.0.3',
    ]
    return versions[versions.index(from_version) + 1:]


# def migration__0_0_4():
#     models.db.engine.execute("ALTER TABLE config ADD COLUMN foo INT;")


def run_migrations(from_version):
    step_list = get_step_list(from_version)
    for step in step_list:
        migration_func = globals()['migration__' + step.replace('.', '_')]
        logger.warning("Migration from v%s to v%s...", from_version, step)
        migration_func()
        models.Config.get().version = step
        models.db.session.commit()
        from_version = step



#!/usr/bin/env python
# -*- coding:utf-8 -*-

from __future__ import unicode_literals, print_function

import datetime

from models import Feed, Entry
from fetcher import concat_urls
from sys import version_info as python_version
if python_version[0] == 3:
    basestring = (bytes, str)

def normalize_feed_dict(dico):
    """
        feedparser.FeedParserDict from different sites may differ
        in their attributes : we have to normalize that.
    """

    norm = {}
    assert isinstance(dico, dict)
    f = dico['feed']
    e = dico['entries']
    f_id = f.get('id')
    if isinstance(f_id, basestring) and f_id[:4] == 'http':
        norm['url'] = f_id
    else:
        norm['url'] = f.get('link')
    norm['title'] = f.get('title') or f.get('id')
    norm['link'] = f.get('link')
    if not norm['link'].startswith('http'):
        norm['link'] = 'http://' + norm['link']
    norm['subtitle'] = f.get('subtitle')
    norm['author'] = f.get('author')
    norm['generator'] = f.get('generator')
    norm['encoding'] = dico.get('encoding')
    structTime = f.get('updated_parsed')
    try:
        norm['updated'] = datetime.datetime(*structTime[:6])
    except (TypeError, ValueError) as err:
        norm['updated'] = None
    norm['entries_hash'] = id(e) # TODO hash
    return norm


def normalize_entry_dict(dico, feed_url):
    """
        feedparser.FeedParserDict from different sites may differ
        in their attributes : we have to normalize that.
    """

    norm = {}
    assert isinstance(dico, dict)
    e_id = dico.get('id')
    if isinstance(e_id, basestring) and e_id[:4] == 'http':
        norm['url'] = e_id
    else:
        norm['url'] = dico.get('link')
    if not norm['url'].startswith('http'):
        norm['url'] = concat_urls(feed_url, norm['url'])
    norm['link'] = dico.get('link')
    if not norm['link'].startswith('http'):
        norm['link'] = concat_urls(feed_url, norm['link'])
    norm['title'] = dico.get('title')
    norm['content'] = dico.get('value')
    if norm['content'] is None:
        try:
            norm['content'] = dico['content'][0]['value']
        except (KeyError, IndexError) as err:
            norm['content'] = dico.get('summary')
    try:
        norm['mimetype'] = dico['content'][0]['type']
    except (KeyError, IndexError) as err:
        norm['mimetype'] = None
    structTime = dico.get('published_parsed')
    try:
        norm['created'] = datetime.datetime(*structTime[:6])
    except (TypeError, ValueError) as err:
        norm['created'] = None
    structTime = dico.get('updated_parsed')
    try:
        norm['updated'] = datetime.datetime(*structTime[:6])
    except (TypeError, ValueError) as err:
        norm['updated'] = None
    if norm['created'] is None and norm['updated'] is not None:
        norm['created'] = norm['updated']
    elif norm['updated'] is None and norm['created'] is not None:
        norm['updated'] = norm['created']
    return norm


def FeedFromDict(dico, config):
    dico = normalize_feed_dict(dico)
    feed = Feed(dico['url'])
    feed.title = dico['title']
    feed.link = dico['link']
    feed.subtitle = dico['subtitle']
    feed.author = dico['author']
    feed.generator = dico['generator']
    feed.encoding = dico['encoding']
    feed.updated = dico['updated']
    feed.entries_hash = dico['entries_hash']
    feed.refresh_interval = config.default_refresh_interval
    feed.max_entries = config.default_max_entries
    feed.highlight_news = config.default_highlight_news
    feed.entries_per_page = config.default_entries_per_page
    feed.has_news = False
    return feed


def EntryFromDict(dico, feed_url):
    dico = normalize_entry_dict(dico, feed_url)
    entry = Entry(dico['title'], dico['content'])
    entry.url = dico['url']
    entry.link = dico['link']
    entry.title = dico['title']
    entry.content = dico['content']
    entry.mimetype = dico['mimetype']
    entry.created = dico['created']
    entry.updated = dico['updated']
    entry.public = False
    return entry



#!/usr/bin/env python
# -*- coding:utf-8 -*-

from __future__ import unicode_literals, print_function
import sys

from flask import Flask
from flask.ext.babel import Babel

def current_module():
	return sys.modules[__name__]

def create_flask_app(prefix):
	current_module().app = Flask(__name__, static_url_path=prefix + "/static")
	create_babel()
	return current_module().app

def create_babel():
	current_module().babel = Babel(current_module().app)



#!/usr/bin/env python
# -*- coding:utf-8 -*-

from setuptools import setup, find_packages
import zenfeed

setup(
    name="zenfeed",
    version=zenfeed.__version__,
    install_requires=["beautifulsoup4", "Flask", "Flask-SQLAlchemy", "docopt",
                      "feedparser", "gevent", "path.py", "requests",
                      "flask-babel", "pytz"],
    packages=find_packages(),
    author="fspot",
    author_email="fred@fspot.org",
    description="Zen RSS feed reader.",
    long_description=open("README.md").read(),
    include_package_data=True,
    zip_safe=False,
    url="http://github.com/fspot/zenfeed",
    entry_points = {
        "console_scripts": [
            'zenfeed = zenfeed.start:main',
        ],
    },
    license="BSD",
)




#!/usr/bin/env python
# -*- coding:utf-8 -*-

from __future__ import unicode_literals, print_function
import gevent
from gevent.queue import Queue

from actor import Actor
from log import logger
from models import Feed
from workers import (new_feed_worker, deadline_worker,
    cache_worker, delete_worker)

class DeadlineManager(Actor):
    def __init__(self):
        self.workers = {}
        Actor.__init__(self)

    def launch_deadline_worker(self, feed):
        assert self.workers.get(feed.id) is None
        inbox = Queue()
        w = gevent.spawn(deadline_worker, feed, inbox, self.inbox)
        self.workers[feed.id] = {
            'feed': feed,
            'worker': w,
            'queue': inbox,
        }

    def launch_deadline_workers(self, feeds):
        for feed in feeds:
            self.launch_deadline_worker(feed)

    def on_message(self, message, answer_box=None):
        """
            Message type can be :
            - new-feed
            - force-refresh-feed
            - delete-feed
            - edit-refresh-interval-feed
            - refresh-cache
        """

        assert isinstance(message, dict)
        msg_type = message.get('type')
        if msg_type == 'new-feed':
            logger.warning('>>> new-feed request : %s', message['url'])
            gevent.spawn(new_feed_worker, message['url'],
                         self.favicon_dir, answer_box, self.inbox)
        elif msg_type == "new-deadline-worker":
            logger.warning('>>> new-deadline-worker request : %d', message['feed_id'])
            feed = Feed.query.get(message['feed_id'])
            self.launch_deadline_worker(feed)
        elif msg_type == "force-refresh-feed":
            logger.warning('>>> force-refresh-feed request : %d', message['id'])
            self.workers[message['id']]['queue'].put({
                'type': 'force-refresh',
                'answer_box': answer_box,
            }) # TODO put Mail instead ?
        elif msg_type == "refresh-cache":
            gevent.spawn(cache_worker, message.get('feed_id'))
        elif msg_type == "delete-feed":
            worker = self.workers.get(message['feed_id'])
            if worker:
                del self.workers[message['feed_id']]
            gevent.spawn(delete_worker, worker, message['feed_id'], answer_box)
        else:
            logger.error("Unknown message type : %s", msg_type)


deadlineManager = DeadlineManager()



#!/usr/bin/env python
# -*- coding:utf-8 -*-

from __future__ import unicode_literals, print_function
from datetime import datetime
from functools import wraps
from hashlib import sha256
from math import ceil
from flask import (render_template, send_from_directory, request,
                   redirect, url_for, session, jsonify)
from gevent.queue import Queue
from flask.ext.babel import format_datetime

from actor import Mail
from app import app, babel
from log import logger
from models import db, Tag, Feed, Entry, Config
from deadline_manager import deadlineManager
from settings import LANGUAGES

PRE = app.config['PATH_PREFIX']

def need_root(vue):
    @wraps(vue)
    def decorated(*args, **kwargs):
        if not Config.validate_password(session.get('pw')):
            return redirect(url_for('login', next=request.path))
        return vue(*args, **kwargs)
    return decorated

def refresh_highlighted_feeds(vue):
    @wraps(vue)
    def decorated(*args, **kwargs):
        feed_id = kwargs['feed_id']
        if not kwargs.get('bot_flag'):
            feed = Feed.query.get(feed_id)
            if feed.has_news:
                feed.has_news = False
                db.session.commit()
                deadlineManager.inbox.put(
                    {'type': 'refresh-cache', 'feed_id': None}
                )
        return vue(*args, **kwargs)
    return decorated

# main site routes :

@app.route(PRE + '/')
def index():
    if app.cache.has('/'): return app.cache.get('/')
    logger.info("Index view is processing : will be cached")
    feeds = Feed.query.order_by(Feed.updated.desc())
    resp = render_template('feeds.html', feeds=feeds)
    return app.cache.put(resp, '/')

@app.route(app.static_url_path + '/favicon/<favicon>')
def get_favicon(favicon):
    return send_from_directory(app.config['FAVICON_DIR'], favicon)

@app.route(PRE + '/<int:feed_id>/')
@app.route(PRE + '/<int:feed_id>/page-<int:page>')
@refresh_highlighted_feeds
def feed_view(feed_id, page=1, bot_flag=False):
    if page == 1 and app.cache.has(feed_id=feed_id):
        return app.cache.get(feed_id=feed_id)
    logger.info("Feed [%d] view is processing : will be cached", feed_id)
    feed = Feed.query.get(feed_id)
    entries = feed.entries.order_by(
        Entry.updated.desc()
    ).offset(feed.entries_per_page * (page - 1)).limit(feed.entries_per_page)
    nb_pages = int(ceil(feed.entries.count() / float(feed.entries_per_page)))
    resp = render_template(
        'feed.html', feed=feed, entries=entries, page=page, nb_pages=nb_pages
    )
    if page == 1:
        app.cache.put(resp, feed_id=feed_id)
    return resp

@app.route(PRE + '/<int:feed_id>/<int:entry_id>/')
def entry_view(feed_id, entry_id):
    entry = Entry.query.get(entry_id)
    return render_template('entry.html', entry=entry)

# login / logout :

@app.route(PRE + '/login/', methods=['GET', 'POST'])
def login():
    next_page = request.args.get('next') or url_for('index')
    if request.method == 'POST':
        session['pw'] = Config.hash(request.form['pw'])
        return redirect(next_page)
    return render_template('login.html', next_page=next_page)

@app.route(PRE + '/logout/')
def logout():
    next_page = request.args.get('next') or url_for('index')
    session.pop('pw', None)
    return redirect(next_page)

# admin and api :

@app.route(PRE + '/admin/')
@need_root
def admin_index():
    return render_template('admin.html', path_prefix=PRE)

@app.route(PRE + '/admin/<filename>.html')
@need_root
def admin_view(filename):
    return redirect(app.static_url_path + '/html/' + filename + '.html')

@app.route(PRE + '/api/config/')
@need_root
def api_get_config():
    config = Config.get()
    return jsonify(config.to_dict(exclude_fields=['password', 'id']))

@app.route(PRE + '/api/config/', methods=['POST'])
@need_root
def api_edit_config():
    config = Config.get()
    # update all fields, no checks (TODO)
    config_fields = config.fields_set(exclude_fields=['id', 'password'])
    fields = config_fields & set(request.json.keys())
    for field in fields:
        setattr(config, field, request.json[field])
    # change password if the "pw" field isnt empty
    pw = request.json.get('pw')
    if pw:
        Config.change_password(pw)
    db.session.merge(config)
    db.session.commit()
    Config.refresh_instance()  # not necessary ?
    return jsonify({'msg': 'Success !'})

def clean_feed_to_dict(feed):
    f = feed.to_dict(exclude_fields=['entries', 'tags', 'entries_hash'])
    f['updated'] = int(feed.updated.strftime("%s")) if feed.updated else None
    f['favicon_url'] = url_for('get_favicon', favicon=feed.favicon_path)
    f['nb_of_entries'] = feed.entries.count()
    return f

@app.route(PRE + '/api/feed/')
@need_root
def api_get_feed():
    feeds = Feed.query.order_by(Feed.updated.desc())
    ret = {'feeds': []}
    for feed in feeds:
        f = clean_feed_to_dict(feed)
        ret['feeds'].append(f)
    return jsonify(ret)

@app.route(PRE + '/api/feed/', methods=['POST'])
@need_root
def api_edit_feed():
    feed_id = request.json['id']
    feed = Feed.query.get(feed_id)
    # update some fields, no checks (TODO)
    feed_fields = {
        'url', 'title', 'link', 'refresh_interval',
        'max_entries', 'entries_per_page', 'highlight_news',
    }
    fields = feed_fields & set(request.json.keys())
    for field in fields:
        setattr(feed, field, request.json[field])
    db.session.merge(feed)
    db.session.commit()
    deadlineManager.inbox.put(
        {'type': 'refresh-cache', 'feed_id': feed_id}
    )
    return jsonify({'msg': 'Success !'})

@app.route(PRE + '/api/feed/add/', methods=['POST'])
@need_root
def api_add_feed():
    url = request.json['url']
    inbox = Queue()
    mail = Mail(inbox, {'type': 'new-feed', 'url': url})
    deadlineManager.inbox.put(mail)
    feed = inbox.get()
    dictfeed = clean_feed_to_dict(feed)
    return jsonify({"msg": feed.title, 'feed': dictfeed})

@app.route(PRE + '/api/feed/delete/', methods=['POST'])
@need_root
def api_delete_feed():
    feed_id = request.json['id']
    inbox = Queue()
    mail = Mail(inbox, {'type': 'delete-feed', 'feed_id': feed_id})
    deadlineManager.inbox.put(mail)
    resp = inbox.get()
    return jsonify({"msg": resp['success']})

# Babel

@babel.localeselector
def get_locale():
    return (app.config['FIXED_LANGUAGE'] or
            request.accept_languages.best_match(LANGUAGES.keys()))

@babel.timezoneselector
def get_timezone():
    return app.config['FIXED_TIMEZONE']

# Jinja

@app.template_filter('humanize_date')
def _jinja2_humanize_datetime(date, locale=None):
    if date is not None:
        fmt = "d/MM, H':'mm"
        if datetime.now().year != date.year:
            fmt = "d/MM/yyyy"
        return format_datetime(date, fmt)
